import { h } from 'preact';
import { withText, Text } from 'preact-i18n';
import * as styles from './panel.scss';
import * as globalStyles from '../../styles/global.scss';
import { useMemo, useRef, useState } from 'preact/hooks';
import { SearchField } from '../search-field/search-field';
import { AccuracyLabel } from './accuracy-label';
import { ClosePanel } from '../icons/close-panel';
import { TranscriptContainer } from './transcript-container';
import { MentionedTerms } from './mentioned-terms';
import { VerificationStatus } from '../../types/verification-status';
import type { DataProvider } from '../../providers/data-provider';
import type { CuePoint } from '../../types/cuepoint';

const translations = {
  closePanelButton: <Text id="taxonomyTranscript.closePanelButton">Close Transcript Panel</Text>
};

interface Props {
  onCloseButtonClick: () => void;
  dataProvider: DataProvider;
  isActive: boolean;
  player: KalturaPlayerTypes.Player;
  activeCuePoint: CuePoint | null;
}
export const Panel = withText(translations)(({
  closePanelButton,
  onCloseButtonClick,
  dataProvider,
  isActive,
  player,
  activeCuePoint
}: Props & { closePanelButton: string }) => {
  const [searchValue, setSearchValue] = useState<string>('');
  const [activeTerm, setActiveTerm] = useState<string | null>(null);
  const transcriptPanelContainerRef = useRef<HTMLDivElement>(null);

  const handleKeywordChange = (value: string): void => {
    setSearchValue(value);
    setActiveTerm(null);
  };

  function handleTermClick(term: string): void {
    if (activeTerm === term) {
      // reset
      setSearchValue('');
      setActiveTerm(null);

      return;
    }

    setSearchValue(term);
    setActiveTerm(term);
  }

  if (!isActive) {
    return null;
  }

  function handleTranscriptTextClick(time: number): void {
    player.currentTime = time;
  }

  const verificationStatus = dataProvider.getVerificationStatus();
  const captionsAsCuePoints = useMemo(() => {
    return dataProvider.getCaptionsCuePoints();
  }, [dataProvider]);
  const mentionedTerms = dataProvider.getMentionedTerms();

  return (
    <div ref={transcriptPanelContainerRef} className={styles.transcriptPanelContainer}>
      <div className={styles.searchAndCloseButtonRow}>
        <SearchField searchValue={searchValue} onChange={handleKeywordChange} />
        <button className={styles.closeButton} onClick={onCloseButtonClick}>
          <span aria-hidden>
            <ClosePanel />
          </span>
          <span className={globalStyles.visuallyHidden}>{closePanelButton}</span>
        </button>
      </div>
      <MentionedTerms activeTerm={activeTerm} terms={mentionedTerms} onTermClick={handleTermClick} />
      {verificationStatus !== VerificationStatus.Empty ? <AccuracyLabel verificationStatus={verificationStatus} /> : null}
      <TranscriptContainer
        activeCuePoint={activeCuePoint}
        cuePoints={captionsAsCuePoints}
        highlightedWord={activeTerm || searchValue}
        transcriptContainerRef={transcriptPanelContainerRef}
        onTextLineClick={handleTranscriptTextClick}
      />
    </div>
  );
});
